import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
from graph_tool.all import *




class TestWindow(Gtk.Window):
	def __init__(self):
			Gtk.Window.__init__(self, title="Test")
			self.resize(800, 600)
			#self.button = Gtk.Button(label="Display fucking graph")
			#self.button.connect("clicked", self.on_button_clicked)
			#self.add(self.button)
			
	#def on_button_clicked(self, widget):
			g = Graph()
			v1 = g.add_vertex()
			v2 = g.add_vertex()
			e = g.add_edge(v1, v2)
			
			
			nb = Gtk.Notebook()
			self.add(nb)
			
			page1 = Gtk.Box()
			nb.append_page(page1, Gtk.Label("Page 1"))
			
			
			
			position = sfdp_layout(g)
			l = GraphWidget(g, pos=position)
			
			nb.append_page(l, Gtk.Label("Graph"))
			
			#grid = Gtk.Grid()
			
			#self.add(l2)
			#grid.add(l)
			
			
class TestWindow2(Gtk.Window):
	def __init__(self):
			Gtk.Window.__init__(self, title="Test")
			self.resize(800, 600)
			
			self.grid = Gtk.Grid()
			self.add(self.grid)
			
			self.button = Gtk.Button(label="Display fucking graph")
			self.button.connect("clicked", self.on_button_clicked)
			self.grid.add(self.button)
			
	def on_button_clicked(self, widget):
			g = Graph()
			v1 = g.add_vertex()
			v2 = g.add_vertex()
			v3 = g.add_vertex()
			v4 = g.add_vertex()
			v5 = g.add_vertex()
			v6 = g.add_vertex()
			e1 = g.add_edge(v1, v2)
			e2 = g.add_edge(v2, v3)
			e3 = g.add_edge(v1, v3)
			e4 = g.add_edge(v1, v5)
			e5 = g.add_edge(v2, v4)
			e6 = g.add_edge(v1, v6)
			
			position = sfdp_layout(g)
			l = GraphWidget(g, pos=position, vertex_text=g.vertex_index, vertex_font_size=15)
			l.set_size_request(400,400)
			self.grid.attach(l, 0, 1, 5, 5)
			window.show_all()
			#grid.add(l)
			
window = TestWindow2()
window.connect("delete-event", Gtk.main_quit)
window.show_all()
Gtk.main()



# -*- coding: utf-8 -*-
import time

import galaxies.src.core.database_tools as db


class Noeud:
    """ Permet d'énumérer les noeuds du graphe """
    def __init__(self, step_time):
        self.val = 0
        self.step_time = step_time
        self.tempsIni = time.clock()

    def nouvelle_valeur(self):
        self.val += 1

        # Affichage du temps de construction
        if divmod(self.val, self.step_time)[1] == 0:
            self.temps = time.clock()
            print("Nombre noeuds du graphe: " + str(self.val) +"; temps de construction du graphe pour les " + str(self.step_time) + " derniers noeuds: " + str(self.temps - self.tempsIni))
            self.tempsIni = self.temps


def construction_graphe(database_path: str, step_time: int):
    # Connexion to the database
    connexion = db.connexion(database_path)
    cursor = connexion.cursor()

    # Retrieve all the books and stack them in a node
    node = Noeud(step_time=step_time)
    cursor.execute('''SELECT rowid FROM livres''')
    for id_reference in cursor:
        fusion_sources_cibles(id_reference, connexion, node)

    # Store
    cursor.execute('''INSERT INTO maxNoeud values (?)''', (node.val,))
    connexion.commit()
    connexion.close()
    return node.val


def fusion_sources_cibles(id_ref, connexion, node):
    curseur_source = connexion.cursor()
    curseur_cible = connexion.cursor()
    curseur_source.execute("""
        SELECT ordonneeSource, empanSource, rowid
        FROM grapheReutilisations
        WHERE idRefSource = ?""", id_ref)
    curseur_cible.execute("""
        SELECT ordonneeCible, empanCible, rowid
        FROM grapheReutilisations
        WHERE idRefCible = ?""", id_ref)

    liste_reutilisation_source = curseur_source.fetchall()
    liste_reutilisation_cible = curseur_cible.fetchall()

    liste_reutilisation_marquee = [(X[0], X[1], -X[2]) for X in liste_reutilisation_cible] + liste_reutilisation_source
    liste_reutilisation_marquee.sort()
    resultat = fusion(liste_reutilisation_marquee, node)
    if resultat:
        node.nouvelle_valeur()
    for X in resultat:
        id_reutilisation, id_noeud = X[2], X[3]
        if X[-2] > 0:
            ajout_source(id_reutilisation, id_noeud, curseur_source)
        elif X[-2] < 0:
            ajout_cible(abs(id_reutilisation), id_noeud, curseur_cible)
        else:
            print("Attention, erreur fusion sur noeud", node.val, "avec X=", X)


def fusion(liste_reutilisation, node):
    if liste_reutilisation == []:
        return []
    elif len(liste_reutilisation) == 1:
        return [[liste_reutilisation[0][0], liste_reutilisation[0][1], liste_reutilisation[0][2], node.val]]
    else:
        tete, suivant, *entree = liste_reutilisation
        resultat = []

        while len(entree) >= 1:
            if tete[0] + tete[1] >= suivant[0]:
                nombre_reutilisations = [tete[0], max(tete[1], suivant[1] - tete[0] + suivant[0]), tete[2], node.val]
                resultat = resultat + [nombre_reutilisations]
                tete = [tete[0], max(tete[1], suivant[1] - tete[0] + suivant[0]), suivant[2]]
                suivant = entree.pop(0)
            else:
                nombre_reutilisations = [tete[0], tete[1], tete[2], node.val]
                resultat = resultat + [nombre_reutilisations]
                tete = suivant
                suivant = entree.pop(0)
                node.nouvelle_valeur()
        else:
            if tete[0] + tete[1] >= suivant[0]:
                nombre_reutilisations = [tete[0], max(tete[1], suivant[1] - tete[0] + suivant[0]), tete[2], node.val]
                resultat = resultat + [nombre_reutilisations]
                tete = [tete[0], max(tete[1], suivant[1] - tete[0] + suivant[0]), suivant[2]]
            else:
                nombre_reutilisations = [tete[0], tete[1], tete[2], node.val]
                resultat = resultat + [nombre_reutilisations]
                tete = suivant
                node.nouvelle_valeur()

        return resultat + [[tete[0], tete[1], tete[2], node.val]]


def ajout_source(id_reutilisation, id_noeud, cursor):
    cursor.execute('''INSERT INTO grapheGalaxiesSource values (?, ?)''',
                   (id_reutilisation, id_noeud))


def ajout_cible(id_reutilisation, id_noeud, cursor):
    cursor.execute('''INSERT INTO grapheGalaxiesCible values (?, ?)''',
                   (id_reutilisation, id_noeud))


def sauvegarde_graphe(database_path: str, step_time: int):
    connexion = db.connexion(database_path)
    curseur_arc = connexion.cursor()
    curseur_noeud = connexion.cursor()
    curseur_graphe = connexion.cursor()

    curseur_arc.execute('''SELECT * FROM maxNoeud''')
    maxNoeud = curseur_arc.fetchone()[0]
    n = Noeud(step_time=step_time)
    while n.val < maxNoeud:
        curseur_arc.execute('''SELECT idReutilisation FROM grapheGalaxiesSource WHERE idNoeud = (?)''', (n.val,))
        reutilisation = curseur_arc.fetchone()
        while reutilisation != None:
            curseur_noeud.execute('''SELECT idNoeud FROM grapheGalaxiesCible WHERE idReutilisation = (?)''', (reutilisation[0],))
            nouveau_noeud = curseur_noeud.fetchone()
            curseur_graphe.execute('''INSERT INTO grapheGalaxies values (?,?)''', (n.val, nouveau_noeud[0], ))
            reutilisation = curseur_arc.fetchone()
        n.nouvelle_valeur()
    n = Noeud(step_time=step_time)
    while n.val < maxNoeud:
        curseur_arc.execute('''SELECT idReutilisation FROM grapheGalaxiesCible WHERE idNoeud = (?)''', (n.val,))
        reutilisation = curseur_arc.fetchone()
        while reutilisation != None:
            curseur_noeud.execute('''SELECT idNoeud FROM grapheGalaxiesSource WHERE idReutilisation = (?)''', (reutilisation[0],))
            nouveau_noeud = curseur_noeud.fetchone()
            curseur_graphe.execute('''INSERT INTO grapheGalaxies values (?,?)''', (n.val, nouveau_noeud[0],))
            reutilisation = curseur_arc.fetchone()
        n.nouvelle_valeur()
    connexion.commit()
    connexion.close()


if __name__ == '__main__':
    pass



import gi

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk
from graph_tool.all import *


class Gui:
    def __init__(self):
        self.window = Gtk.Window(title="Test")
        # self.window.set_size_request(500, 500)
        # self.window.set_position(Gtk.WIN_POS_CENTER)

        # self.grid = Gtk.Grid()

        self.menubar = Gtk.MenuBar()

        self.menu1 = Gtk.Menu()
        self.menu1_file = Gtk.MenuItem("Fichier")
        self.menu1_file.set_submenu(self.menu1)

        self.menu2 = Gtk.Menu()
        self.menu2_filtre = Gtk.MenuItem("Filtre")
        self.menu2_filtre.set_submenu(self.menu2)

        self.menu3 = Gtk.Menu()
        self.menu3_aide = Gtk.MenuItem("Aide")
        self.menu3_aide.set_submenu(self.menu2)

        self.charger_reutilisation = Gtk.MenuItem("Charger Réutilisation")
        self.menu1.append(self.charger_reutilisation)
        self.charger_graphe = Gtk.MenuItem("Charger Graphe")
        self.menu1.append(self.charger_graphe)
        self.menu1.append(Gtk.SeparatorMenuItem())
        self.save_graphe = Gtk.MenuItem("Sauvegarder Graphe")
        self.menu1.append(self.save_graphe)
        self.menu1.append(Gtk.SeparatorMenuItem())
        self.exit = Gtk.MenuItem("Exit")
        self.menu1.append(self.exit)

        self.filtre_auteur = Gtk.MenuItem("Filtrer Auteur")
        self.menu2.append(self.filtre_auteur)
        self.filtre_annee = Gtk.MenuItem("Filtrer Année")
        self.menu2.append(self.filtre_annee)

        self.menubar.append(self.menu1_file)
        self.menubar.append(self.menu2_filtre)
        self.menubar.append(self.menu3_aide)

        self.vbox = Gtk.VBox(False, 2)
        self.vbox.pack_start(self.menubar, False, False, 0)
        self.window.add(self.vbox)

        self.hbox = Gtk.HBox(False, 0)

        self.sw = Gtk.ScrolledWindow()
        self.sw2 = Gtk.ScrolledWindow()

        self.vbox2 = Gtk.VBox(False, 2)

        self.but = Gtk.Button(label="Filtre 1\nFiltre Auteur: Shakespear")
        self.but2 = Gtk.Button(label="Filtre 2\nFiltre Auteur: Flaubert")
        self.but3 = Gtk.Button(label="Filtre 3\nFiltre Auteur: Maupassant")

        self.but4 = Gtk.Button(label="Filtre 3\nFiltre Auteur: Shakespear")
        self.but5 = Gtk.Button(label="Filtre 4\nFiltre Auteur: Flaubert")
        self.but6 = Gtk.Button(label="Filtre 5\nFiltre Auteur: Maupassant")

        self.but7 = Gtk.Button(label="Filtre 6\nFiltre Auteur: Shakespear")
        self.but8 = Gtk.Button(label="Filtre 7\nFiltre Auteur: Flaubert")
        self.but9 = Gtk.Button(label="Filtre 8\nFiltre Auteur: Maupassant")

        self.label = Gtk.Label(
            label="BlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablabl\nablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablabla\nBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablabla")

        self.sw.set_size_request(200, 500)
        self.sw2.set_size_request(200, 150)

        g = Graph()
        v1 = g.add_vertex()
        v2 = g.add_vertex()
        v3 = g.add_vertex()
        v4 = g.add_vertex()
        v5 = g.add_vertex()
        v6 = g.add_vertex()
        e1 = g.add_edge(v1, v2)
        e2 = g.add_edge(v2, v3)
        e3 = g.add_edge(v1, v3)
        e4 = g.add_edge(v1, v5)
        e5 = g.add_edge(v2, v4)
        e6 = g.add_edge(v1, v6)

        position = sfdp_layout(g)
        l = GraphWidget(g, pos=position, vertex_text=g.vertex_index,
                        vertex_font_size=15)
        l.set_size_request(600, 500)
        self.vbox2.pack_start(self.but, False, True, 5)
        self.vbox2.pack_start(self.but2, False, True, 5)
        self.vbox2.pack_start(self.but3, False, True, 5)

        self.vbox2.pack_start(self.but4, False, True, 5)
        self.vbox2.pack_start(self.but5, False, True, 5)
        self.vbox2.pack_start(self.but6, False, True, 5)

        self.vbox2.pack_start(self.but7, False, True, 5)
        self.vbox2.pack_start(self.but8, False, True, 5)
        self.vbox2.pack_start(self.but9, False, True, 5)

        self.sw.add_with_viewport(self.vbox2)
        self.hbox.pack_start(self.sw, True, True, 0)
        self.hbox.pack_start(l, False, False, 0)

        color = Gdk.color_parse('#d3d3d3')
        color2 = Gdk.color_parse('#595959')
        self.sw.modify_bg(Gtk.StateType.NORMAL, color)
        self.sw2.modify_bg(Gtk.StateType.NORMAL, color)
        self.sw2.add_with_viewport(self.label)
        self.vbox.add(self.hbox)
        self.vbox.add(self.sw2)

        self.window.connect("delete-event", Gtk.main_quit)
        self.window.show_all()
        Gtk.main()


gui = Gui()



""" This module contains commons methods which has no dependencies (except for
the base python library) and thus are usable in every part of the software.

The methods included in this modules are all file-related functions.
"""
import mmap
import os
import os.path


def line_count(file_path: str) -> int:
    """ Efficiently count the number of lines in a file.
    If possible, the UNIX command `wc -l` must be used instead (as its much
    faster).
    """
    f = open(file_path, "r+")
    buf = mmap.mmap(f.fileno(), 0)
    lines = 0
    readline = buf.readline
    while readline():
        lines += 1
    return lines


def delete_existing_files(*file_pathes):
    """ Delete all files passed as parameters. """
    for filepath in file_pathes:
        if os.path.isfile(filepath):
            os.remove(filepath)


if __name__ == "__main__":
    pass



from tkinter import *
import os

from galaxies.src.gui.galaxie_creation import WindowsGalaxieCreation
from galaxies.src.gui.galaxie_vizualisation import WindowsGalaxieVizualisation


PATH_FILE_LOGO_SORBONNE = os.path.join(os.path.dirname(__file__),
                                       "../../res/logo_sorbonne.gif")
PATH_FILE_LOGO_LIP6 = os.path.join(os.path.dirname(__file__),
                                   "../../res/logo_lip6.gif")


class WindowMainMenu:
    def __init__(self):
        self.window = Tk()
        self.window.geometry("800x400")
        self.window.overrideredirect(True)

        self.quit_button = Button(self.window, text="X",
                                  command=self.window.quit, relief=FLAT,
                                  foreground='red',
                                  font=("Helvetica", 10, "bold"))
        self.quit_button.grid(row=0, columnspan=2, sticky=NE, padx=5)

        self.title = Label(text="GALAXIES",
                           font=("Helvetica", 25, "bold italic"),
                           fg='#%02x%02x%02x' % (29, 40, 105))
        self.title.grid(row=1, columnspan=2, pady=15)

        self.create_button = Button(self.window, text="Create a galaxie",
                                    width=30, height=5,
                                    fg='#%02x%02x%02x' % (29, 40, 105),
                                    command=self.open_create)
        self.explore_button = Button(self.window, text="Explore a galaxie",
                                     width=30, height=5,
                                     fg='#%02x%02x%02x' % (29, 40, 105),
                                     command=self.open_explore)

        self.create_button.grid(row=2, column=0)
        self.explore_button.grid(row=2, column=1)

        self.canvas = Canvas(self.window, width=800, height=200)

        self.logo_sorbonne = PhotoImage(file=PATH_FILE_LOGO_SORBONNE)
        self.logo_lip6 = PhotoImage(file=PATH_FILE_LOGO_LIP6)

        self.canvas.create_image(200, 120, image=self.logo_sorbonne)
        self.canvas.create_image(600, 110, image=self.logo_lip6)

        self.canvas.grid(row=3, columnspan=2)
        self.center(self.window)
        self.window.mainloop()

    def center(self, toplevel):
        """ Center the window at the middle of the screen.
        Source :
        https://stackoverflow.com/a/3353112
        """
        # TODO: Issue with multi-monitors

        toplevel.update_idletasks()
        w = toplevel.winfo_screenwidth()
        h = toplevel.winfo_screenheight()
        size = tuple(
            int(_) for _ in toplevel.geometry().split('+')[0].split('x'))
        x = w / 2 - size[0] / 2
        y = h / 2 - size[1] / 2
        toplevel.geometry("%dx%d+%d+%d" % (size + (x, y)))

    def open_create(self):
        self.quit()
        WindowsGalaxieCreation()

    def open_explore(self):
        self.quit()
        WindowsGalaxieVizualisation()

    def quit(self):
        self.window.destroy()


if __name__ == "__main__":
    pass



# -*- coding: utf-8 -*-
""" Read a source database containg reuses in a corpus, then save them the
useful informations into a SQLite3 database.

The source database is usually the one outputed by the text-align software
(located at https://github.com/ARTFL-Project/text-align), but any CSV file
properly configured can be used.
"""

import codecs
import re

from progressbar import ProgressBar, Percentage, Counter, Timer

import galaxies.src.core.database_tools as db
from galaxies.src.common.file import line_count


PB_WIDGETS = ["Processed: ", Counter(), " lines [", Percentage(), "], ",
              Timer()]

MESSAGE_REUSES_RODE = "{} reuses rode from the source file."


def parse_reuse_file(database_source: str, database_dest: str,
                     max_book_to_add: int, delimiter: str, encoding: str,
                     header: bool, index_src_author: int, index_src_title: int,
                     index_src_date: int, index_src_text_matched: int,
                     index_src_start_byte: int, index_src_end_byte: int,
                     index_dest_author: int, index_dest_title: int,
                     index_dest_date: int, index_dest_text_matched: int,
                     index_dest_start_byte: int, index_dest_end_byte: int):
    """ Read a source database containg reuses in a corpus, then save the
    useful informations into a SQLite3 database.

    The source database is usually the one outputed by the text-align software
    (located at https://github.com/ARTFL-Project/text-align), but any CSV file
    properly configured can be used.
    """
    # Set the number of lines to read for the progress bar
    if max_book_to_add < 0:
        max_book_to_add = line_count(database_source)

    connexion = db.connexion(database_dest)
    curseur = connexion.cursor()
    with codecs.open(database_source, 'r', encoding, errors='replace') as file:
        if header:
            # Remove the header
            file.readline()

        progress_bar = ProgressBar(widgets=PB_WIDGETS, maxval=max_book_to_add)
        progress_bar.start()
        for line_number, line in enumerate(file):
            if line_number == max_book_to_add:
                break

            l = re.split(delimiter, line)
            # Add the two books into the database (do nothing if one of them is
            # already present).
            db.add_book(l[index_src_author],
                        l[index_src_title],
                        l[index_src_date], curseur)
            db.add_book(l[index_dest_author],
                        l[index_dest_title],
                        l[index_dest_date], curseur)

            # Retrieve the identifier of the two books (because the two of them
            # may be already present).
            id_src = db.get_id_book(l[index_src_author],
                                    l[index_src_title],
                                    l[index_src_date],
                                    curseur)
            id_dest = db.get_id_book(l[index_dest_author],
                                     l[index_dest_title],
                                     l[index_dest_date],
                                     curseur)

            # Add a reuse for the source book and the destination book
            real_id_book_source = db.id_ref(l[index_src_author], l[index_src_title], l[index_src_date])
            real_id_book_target = db.id_ref(l[index_dest_author], l[index_dest_title], l[index_dest_date])
            db.add_reuse(id_src,
                         int(l[index_src_start_byte]),
                         int(l[index_src_end_byte]) -
                         int(l[index_src_start_byte]),
                         l[index_src_text_matched],
                         id_dest,
                         int(l[index_dest_start_byte]),
                         int(l[index_dest_end_byte]) -
                         int(l[index_dest_start_byte]),
                         l[index_dest_text_matched],
                         real_id_book_source,
                         real_id_book_target,
                         curseur)
            progress_bar.update(line_number + 1)

    progress_bar.finish()
    connexion.commit()
    connexion.close()

    print(MESSAGE_REUSES_RODE.format(max_book_to_add))


if __name__ == '__main__':
    pass



# -*- coding: utf-8 -*-
import time
import shelve

import galaxies.src.core.database_tools as db


__author__ = 'Jean-Gabriel Ganascia'


class Galaxie:
    """ permet d'énumérer composantes connexes """
    def __init__(self, path_file_galaxies: str, path_file_list_galaxies: str,
                 step_time: int):
        self.val = 0
        self.compositionGalaxie = dict()
        self.tempsIni = time.clock()
        self.step_time = step_time
        self.path_file_galaxies = path_file_galaxies
        self.path_file_list_galaxies = path_file_list_galaxies[:-3]

    def nouvelle_valeur(self):
        self.val += 1
        if divmod(self.val, self.step_time)[1] == 0:
            self.temps = time.clock()
            print("Nombre galaxies: " + str(self.val) +"; temps de construction des " + str(self.step_time) + " dernières galaxies: " + str(self.temps - self.tempsIni))
            self.tempsIni = self.temps

    def noeuds_galaxie(self, n, L):
        self.compositionGalaxie[n] = L
        return len(L)

    def save(self):
        print(self.path_file_list_galaxies)
        dict = shelve.open(self.path_file_list_galaxies)
        x = 0
        while x < self.val:
            dict[str(x)] = self.compositionGalaxie[x]
            x += 1
        dict['nbreGalaxies'] = self.val
        dict.close()

    def rangement(self):
        tr = time.clock()
        print("         Extraction des galaxies terminées; opérations de rangement...")
        connexion = db.connexion(self.path_file_galaxies)
        curseur2 = connexion.cursor()
        curseur2.execute('''INSERT INTO nombreGalaxies values (?)''', (str(self.val),))
        print("Nombre de galaxies: "+str(self.val))
        i = 0
        while i < self.val:
            lnoeuds = self.compositionGalaxie[i]
            n = len(lnoeuds)
            longueur = 0
            for texte in texte_galaxie(i, curseur2, self.path_file_list_galaxies + '.db'):
                longueur += len(texte)
            curseur2.execute('''DELETE from degreGalaxies WHERE idGalaxie = ?''', (str(i),))
            curseur2.execute('''INSERT INTO degreGalaxies values (?,?,?,?)''', (str(i),str(n),str(longueur),str(int(longueur/n)),))
            i += 1
        connexion.commit()
        connexion.close()
        trf = time.clock()
        print("             ... fin des opérations de rangements. Durée: "+str(trf - tr)+" secondes.")


class NoeudMarques:
    def __init__(self, max):
        self.noeuds = dict()
        self.maxNoeud = max
        n = 0
        while n < max:
            self.noeuds[n] = 'non'
            n += 1

    def noeud_non_visite(self, noeudCourant):
        n = noeudCourant
        while n < self.maxNoeud:
            if self.noeuds[n] == 'non':
                return n
            else:
                n += 1
        return None

    def affectation_galaxie(self, n, g):
        if self.noeuds[n] != 'non':
            print("Erreur sur affectation galaxie au noeud " + str(n) + " - précédente affectation: " + str(self.noeuds[str(n)]))
            return 'erreur'
        else:
            self.noeuds[n] = g.val

    def galaxie(self, n):
        g = self.noeuds[n]
        if g == 'non':
            return 'erreur'
        else:
            return g


def extraction_composantes_connexes(path_file_galaxies: str,
                                    path_file_list_galaxies: str,
                                    path_file_adjacency_graph: str,
                                    path_file_adjacency_graph_transposed: str,
                                    maxNoeud: int,
                                    step_number_of_nodes_galaxie):
    graphe = shelve.open(path_file_adjacency_graph)
    graphe_t = shelve.open(path_file_adjacency_graph_transposed)
    noeuds = NoeudMarques(maxNoeud)
    galaxie = Galaxie(path_file_galaxies, path_file_list_galaxies, step_number_of_nodes_galaxie)
    nouveauNoeud = noeuds.noeud_non_visite(0)
    while nouveauNoeud != None:  # < maxNoeud:
        galaxie.noeuds_galaxie(galaxie.val, composante_connexe(nouveauNoeud, galaxie, graphe, graphe_t, noeuds))
        galaxie.nouvelle_valeur()
        nouveauNoeud = noeuds.noeud_non_visite(nouveauNoeud)
    graphe_t.close()
    graphe.close()
    galaxie.save()
    galaxie.rangement()
    return galaxie


def composante_connexe(N, g, graphe, graphe_t, noeuds):
    E_noeuds = set()
    E_noeuds.add(N)
    noeudsVisites = set()
    while E_noeuds.__len__() != 0 :
        E_noeuds = E_noeuds.difference(noeudsVisites)
        E = E_noeuds.pop()
        if not E in noeudsVisites:
            noeuds.affectation_galaxie(E, g)
            E_noeuds.update(fils(E, graphe, graphe_t))
            noeudsVisites.add(E)

        E_noeuds = E_noeuds.difference(noeudsVisites)
        if E_noeuds.intersection(noeudsVisites) != set():
            print("attention!! Noeud "+str(E))
    return noeudsVisites


def fils(X, graphe, graphe_t):
    return graphe[str(X)] + graphe_t[str(X)]


def extraction_composantes_connexes_(path_file_galaxies, path_file_list_galaxies, maxNoeud, step_number_of_nodes_galaxie):
    # Connexion to the database
    connexion = db.connexion(path_file_galaxies)
    cursor = connexion.cursor()

    noeuds = NoeudMarques(maxNoeud)
    galaxie = Galaxie(path_file_galaxies, path_file_list_galaxies , step_number_of_nodes_galaxie)
    tg1 = time.clock()
    nouveauNoeud = noeuds.noeud_non_visite(0)
    nbre_noeuds = 0
    nbre_noeuds_mod = 0

    while nouveauNoeud != None:  # < maxNoeud:
        nbre_noeuds = nbre_noeuds + galaxie.noeuds_galaxie(galaxie.val, composante_connexe_(nouveauNoeud, galaxie, cursor, noeuds))
        if divmod(nbre_noeuds, step_number_of_nodes_galaxie)[0] > nbre_noeuds_mod:
            nbre_noeuds_mod = divmod(nbre_noeuds, step_number_of_nodes_galaxie)[0]
            tg2 = time.clock()
            print("Nombre total de nœuds traités: "+str(nbre_noeuds)+" - Nombre de galaxies constuites: "+str(galaxie.val)+" - temps: "+str(tg2 - tg1)+ "sec.")
            tg1 = tg2
        galaxie.nouvelle_valeur()
        nouveauNoeud = noeuds.noeud_non_visite(nouveauNoeud)
    galaxie.save()
    galaxie.rangement()
    connexion.close()
    return galaxie


def composante_connexe_(N, g, curseur, noeuds):
    E_noeuds = set()
    E_noeuds.add(N)
    noeudsVisites = set()
    while E_noeuds.__len__() != 0 :
        E = E_noeuds.pop()
        if not E in noeudsVisites:
            noeuds.affectation_galaxie(E, g)
            E_noeuds.update(fils_(E, curseur))
            noeudsVisites.add(E)
    return noeudsVisites


def fils_(X, curseur):
    curseur.execute('''SELECT idNoeudFils FROM grapheGalaxies WHERE idNoeudPere = (?)''', (X,))
    L = []
    for X in curseur.fetchall():
        L.append(X[0])
    return L


def cible(arc, curseur):
    curseur.execute('''SELECT idNoeud FROM grapheGalaxiesCible WHERE idReutilisation = (?)''', (arc,))
    return curseur.fetchall()


def source(arc, curseur):
    curseur.execute('''SELECT idNoeud FROM grapheGalaxiesSource WHERE idReutilisation = (?)''', (arc,))
    return curseur.fetchall()


def textes_galaxie(numero, path, path_file_list_galaxies):
    path_file_list_galaxies = path_file_list_galaxies[:-3] # TODO: Il ne faut JAMAIS le .db dans le nom de fichier
    dirGalaxies = shelve.open(path_file_list_galaxies)

    ListeNoeuds = dirGalaxies[str(numero)]
    dirGalaxies.close()
    connexion = db.connexion(path)
    curseur = connexion.cursor()
    reutilisations = set()
    for Noeud in ListeNoeuds:
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesCible WHERE idNoeud = (?)''', (Noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add(L[0][0])
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesSource WHERE idNoeud = (?)''', (Noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add(L[0][0])
    textes = set()
    for idReutilisation in reutilisations:
        curseur.execute('''SELECT texteSource, idRefSource FROM grapheReutilisations WHERE rowid = (?)''', (str(idReutilisation),))
        textes.add(curseur.fetchall()[0][0])
        curseur.execute('''SELECT texteCible, idRefCible FROM grapheReutilisations WHERE rowid = (?)''', (str(idReutilisation),))
        textes.add(curseur.fetchall()[0][0])
    connexion.close()
    return textes


def texte_galaxie(numero, curseur, path_file_list_galaxies):
    path_file_list_galaxies = path_file_list_galaxies[:-3] # TODO: Il ne faut JAMAIS le .db dans le nom de fichier
    dirGalaxies = shelve.open(path_file_list_galaxies)

    ListeNoeuds = dirGalaxies[str(numero)]
    dirGalaxies.close()
    reutilisations = set()
    for Noeud in ListeNoeuds:
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesCible WHERE idNoeud = (?)''', (Noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add(L[0][0])
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesSource WHERE idNoeud = (?)''', (Noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add(L[0][0])
    textes = set()
    for idReutilisation in reutilisations:
        curseur.execute('''SELECT texteSource, idRefSource FROM grapheReutilisations WHERE rowid = (?)''', (str(idReutilisation),))
        textes.add(curseur.fetchall()[0][0])
        curseur.execute('''SELECT texteCible, idRefCible FROM grapheReutilisations WHERE rowid = (?)''', (str(idReutilisation),))
        textes.add(curseur.fetchall()[0][0])
    return textes


def auteurs_galaxie(numero, path_file_galaxies, path_file_list_galaxies):
    path_file_list_galaxies = path_file_list_galaxies[:-3] # TODO: Il ne faut JAMAIS le .db dans le nom de fichier
    dirGalaxies = shelve.open(path_file_list_galaxies)

    ListeNoeuds = dirGalaxies[str(numero)]
    dirGalaxies.close()
    connexion = db.connexion(path_file_galaxies)
    curseur = connexion.cursor()
    reutilisations = set()
    for Noeud in ListeNoeuds:
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesCible WHERE idNoeud = (?)''', (Noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add(L[0][0])
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesSource WHERE idNoeud = (?)''', (Noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add(L[0][0])
    auteurs = set()
    for idReutilisation in reutilisations:
        curseur.execute('''SELECT idRefSource FROM grapheReutilisations WHERE rowid = (?)''',
                        (str(idReutilisation),))
        t1 = curseur.fetchall()[0]
        curseur.execute('''SELECT auteur FROM livres WHERE rowid = (?)''', (t1[0],))
        auteurs.add(curseur.fetchall()[0][0])
        curseur.execute('''SELECT idRefCible FROM grapheReutilisations WHERE rowid = (?)''',
                        (str(idReutilisation),))
        t1 = curseur.fetchall()[0]
        curseur.execute('''SELECT auteur FROM livres WHERE rowid = (?)''', (t1[0],))
        auteurs.add(curseur.fetchall()[0][0])
    connexion.close()
    return auteurs


def textes_et_references_galaxie(numero, path_file_galaxies, path_file_list_galaxies):
    path_file_list_galaxies = path_file_list_galaxies[:-3] # TODO: Il ne faut JAMAIS le .db dans le nom de fichier
    dirGalaxies = shelve.open(path_file_list_galaxies)

    ListeNoeuds = dirGalaxies[str(numero)]
    dirGalaxies.close()
    connexion = db.connexion(path_file_galaxies)
    curseur = connexion.cursor()
    reutilisations = set()
    for Noeud in ListeNoeuds:
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesCible WHERE idNoeud = (?)''', (Noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add(L[0][0])
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesSource WHERE idNoeud = (?)''', (Noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add(L[0][0])
    textes = set()
    for idReutilisation in reutilisations:
        curseur.execute('''SELECT texteSource, idRefSource FROM grapheReutilisations WHERE rowid = (?)''', (str(idReutilisation),))
        t1 = curseur.fetchall()[0]
        curseur.execute('''SELECT auteur, titre, date FROM livres WHERE rowid = (?)''', (t1[1],))
        textes.add((t1[0],curseur.fetchall()[0]))
        curseur.execute('''SELECT texteCible, idRefCible FROM grapheReutilisations WHERE rowid = (?)''', (str(idReutilisation),))
        t1 = curseur.fetchall()[0]
        curseur.execute('''SELECT auteur, titre, date FROM livres WHERE rowid = (?)''', (t1[1],))
        textes.add((t1[0], curseur.fetchall()[0]))
    connexion.close()
    return textes


def textes_liste_noeuds(ListeNoeuds, path_file_galaxies):
    connexion = db.connexion(path_file_galaxies)
    curseur = connexion.cursor()
    reutilisations = set()
    for noeud in ListeNoeuds:
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesCible WHERE idNoeud = (?)''', (noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add(L[0][0])
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesSource WHERE idNoeud = (?)''', (noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add(L[0][0])
    textes = set()
    for id_reutilisation in reutilisations:
        curseur.execute('''SELECT texteSource, idRefSource FROM grapheReutilisations WHERE rowid = (?)''', (str(id_reutilisation),))
        textes.add(curseur.fetchall()[0][0])
        curseur.execute('''SELECT texteCible, idRefCible FROM grapheReutilisations WHERE rowid = (?)''', (str(id_reutilisation),))
        textes.add(curseur.fetchall()[0][0])
    connexion.close()
    return textes


def textes_et_references_liste_noeuds(ListeNoeuds, path_file_galaxies):
    connexion = db.connexion(path_file_galaxies)
    curseur = connexion.cursor()
    reutilisations = set()
    for Noeud in ListeNoeuds:
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesCible WHERE idNoeud = (?)''', (Noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add(L[0][0])
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesSource WHERE idNoeud = (?)''', (Noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add(L[0][0])
    textes = set()
    for idReutilisation in reutilisations:
        curseur.execute('''SELECT texteSource, idRefSource FROM grapheReutilisations WHERE rowid = (?)''', (str(idReutilisation),))
        t1 = curseur.fetchall()[0]
        curseur.execute('''SELECT auteur, titre, date FROM livres WHERE rowid = (?)''', (t1[1],))
        textes.add((t1[0],curseur.fetchall()[0]))
        curseur.execute('''SELECT texteCible, idRefCible FROM grapheReutilisations WHERE rowid = (?)''', (str(idReutilisation),))
        t1 = curseur.fetchall()[0]
        curseur.execute('''SELECT auteur, titre, date FROM livres WHERE rowid = (?)''', (t1[1],))
        textes.add((t1[0], curseur.fetchall()[0]))
    connexion.close()
    return textes


def textes_et_references_liste_noeuds_avec_noeuds(ListeNoeuds, path_file_galaxies):
    connexion = db.connexion(path_file_galaxies)
    curseur = connexion.cursor()
    reutilisations = set()
    for Noeud in ListeNoeuds:
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesCible WHERE idNoeud = (?)''', (Noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add((L[0][0],Noeud))
        curseur.execute('''SELECT idReutilisation FROM grapheGalaxiesSource WHERE idNoeud = (?)''', (Noeud,))
        L = curseur.fetchall()
        if L != []:
            reutilisations.add((L[0][0], Noeud))
    textes = set()
    for idReutilisation in reutilisations:
        curseur.execute('''SELECT texteSource, idRefSource FROM grapheReutilisations WHERE rowid = (?)''', (str(idReutilisation[0]),))
        t1 = curseur.fetchall()[0]
        curseur.execute('''SELECT auteur, titre, date FROM livres WHERE rowid = (?)''', (t1[1],))
        textes.add((t1[0],curseur.fetchall()[0], idReutilisation[1]))
        curseur.execute('''SELECT texteCible, idRefCible FROM grapheReutilisations WHERE rowid = (?)''', (str(idReutilisation[0]),))
        t1 = curseur.fetchall()[0]
        curseur.execute('''SELECT auteur, titre, date FROM livres WHERE rowid = (?)''', (t1[1],))
        textes.add((t1[0], curseur.fetchall()[0], idReutilisation[1]))
    connexion.close()
    return textes


if __name__ == '__main__':
    pass



import json
import os


_PATH_GLOBAL_VARIABLES = "../../res/global_variables.json"


def _get_value_from_file(value):
    path = os.path.join(os.path.dirname(__file__),
                        _PATH_GLOBAL_VARIABLES)
    with open(path) as file:
        return json.load(file)[value]


def name():
    return _get_value_from_file("name")


def description():
    return _get_value_from_file("description")


def keywords():
    return _get_value_from_file("keywords")


def main_homepage():
    return _get_value_from_file("main_homepage")


def download_url():
    return _get_value_from_file("download_url")


def version():
    return _get_value_from_file("version")


def author():
    return _get_value_from_file("author")


def maintainer():
    return _get_value_from_file("maintainer")


def email():
    return _get_value_from_file("email")


def status():
    return _get_value_from_file("status")


def copyright_text():
    return _get_value_from_file("copyright")


def credits_authors():
    return _get_value_from_file("credits")


def license_used():
    return _get_value_from_file("license")


def classifiers():
    return _get_value_from_file("classifiers")


def dependencies():
    return _get_value_from_file("dependencies")


def cli_entry_point():
    return _get_value_from_file("cli_entry_point")


def gui_entry_point():
    return _get_value_from_file("gui_entry_point")


if __name__ == '__main__':
    pass



import gi
import sqlite3 
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk
from graph_tool.all import *


def get_father_son(path):
	connection = sqlite3.connect(path)
	cursor = connection.cursor()
	cursor.execute("SELECT max(idNoeudPere) FROM grapheGalaxies")
	maxi = cursor.fetchone()
	cursor.execute("SELECT * FROM grapheGalaxies")
	rows = cursor.fetchall()
	connection.close()
	return maxi, rows


def get_authors_date(path):
	connection = sqlite3.connect(path)
	cursor = connection.cursor()
	cursor.execute("SELECT DISTINCT auteur FROM livres")
	authors = cursor.fetchall()
	cursor.execute("SELECT DISTINCT date FROM livres")
	date = cursor.fetchall()
	connection.close()
	return authors, date


def get_filtre(path, choix, inverse, filtre ):
	pass



""" Ce module définit l'interface principale utilisée par la CLI et la GUI.
Les méthodes utilisées ici devront être les seules appelables de l'exterieur.
"""
import time
import os

import galaxies.src.core.parser as parser
import galaxies.src.core.database_tools as database_tools
import galaxies.src.core.graphe_galaxies as graphe_galaxies
import galaxies.src.core.extraction_galaxies as extraction_galaxies
from galaxies.src.common.file import delete_existing_files


def create_reuses_database(*, database_src: str, path_file_galaxies: str,
                           path_file_list_galaxies: str,
                           path_file_adjacency_graph: str,
                           path_file_adjacency_graph_transposed: str,
                           path_dir_dest: str,
                           max_book_to_add: int, delimiter: str,
                           encoding_src: str, remove_header: bool,
                           index_src_author: int,
                           index_src_title: int, index_src_date: int,
                           index_src_text_matched: int,
                           index_src_start_byte: int, index_src_end_byte: int,
                           index_dest_author: int, index_dest_title: int,
                           index_dest_date: int, index_dest_text_matched: int,
                           index_dest_start_byte: int,
                           index_dest_end_byte: int,
                           step_nodes_time: int,
                           step_galaxies_time: int,
                           step_number_of_nodes_galaxie: int):
    """ Create the SQLite3 database containing only the useful information the
    software needs to process the graph.
    :param database_src: The path to the database containing all the reuses'.
    This database must be in the TSV, TAB or CSV format.
    :param path_file_galaxies: The path to the SQLite3 which is gonna be created.
    :param path_file_list_galaxies: The path to the file containing the list of
    all galaxies.
    :param path_file_adjacency_graph: The path of the pickle file containing the
    adjacency graph.
    :param path_file_adjacency_graph_transposed: The path of the pickle file
    containing the transposed of the adjacency graph.
    :param path_dir_dest: The directory containing all the files which'll be
    created by the software.
    :param max_book_to_add: The maximal number of books to add in the SQLite3
    database. If negative, all the books possible are added.
    :param delimiter: The delimiter of the source database containing all the
    reuses'.
    :param encoding_src: The encoding of the source database containing all the
    reuses'.
    :param remove_header: Remove the header present in the source database
    contaning all the reuses'.
    :param index_src_author: The index in the source database matching a column
    containing the author of a source reuse.
    :param index_src_title: The index in the source database matching a column
    containing the title of a source reuse.
    :param index_src_date: The index in the source database matching a column
    containing the date of a source reuse.
    :param index_src_text_matched: The index in the source database matching a
    column containing the text matched of a source reuse.
    :param index_src_start_byte: The index in the source database matching a
    column containing the start byte of a source reuse.
    :param index_src_end_byte: The index in the source database matching a
    column containing the end byte of a source reuse.
    :param index_dest_author: The index in the source database matching a column
    containing the author of a destination (or target) reuse.
    :param index_dest_title: The index in the source database matching a column
    containing the title of a destination (or target) reuse.
    :param index_dest_date: The index in the source database matching a column
    containing the date of a destination (or target) reuse.
    :param index_dest_text_matched: The index in the source database matching a
    column containing the text matched of a destination (or target) reuse.
    :param index_dest_start_byte: The index in the source database matching a
    column containing the start byte of a destination (or target) reuse.
    :param index_dest_end_byte: The index in the source database matching a
    column containing the end byte of a destination (or target) reuse.
    :param step_nodes_time: Print the building time every amount of
    step_nodes_time nodes processed.
    """
    # Delete all existing databases to prevent unwanted errors
    delete_existing_files(path_file_galaxies,
                          path_file_list_galaxies,
                          path_file_adjacency_graph,
                          path_file_adjacency_graph_transposed)

    # Create result dir if it doesn't exists
    os.makedirs(path_dir_dest, exist_ok=True)

    # Creation de la structure de la base
    t1 = time.clock()
    database_tools.create_database(database_path=path_file_galaxies)
    t2 = time.clock()
    print("Temps de construction de la base de données :", t2 - t1, "sec.")

    # Remplissage de la base avec les livres contenus dans le fichier passé en
    # paramètre
    t1 = time.clock()
    parser.parse_reuse_file(database_source=database_src,
                            database_dest=path_file_galaxies,
                            max_book_to_add=max_book_to_add,
                            delimiter=delimiter,
                            encoding=encoding_src,
                            header=remove_header,
                            index_src_author=index_src_author,
                            index_src_title=index_src_title,
                            index_src_date=index_src_date,
                            index_src_text_matched=index_src_text_matched,
                            index_src_start_byte=index_src_start_byte,
                            index_src_end_byte=index_src_end_byte,
                            index_dest_author=index_dest_author,
                            index_dest_title=index_dest_title,
                            index_dest_date=index_dest_date,
                            index_dest_text_matched=index_dest_text_matched,
                            index_dest_start_byte=index_dest_start_byte,
                            index_dest_end_byte=index_dest_end_byte)
    t2 = time.clock()
    print("Temps de lecture du fichier source : " + str(t2 - t1) + "sec.")

    # Création des galaxies
    t1 = time.clock()
    maxNoeud = graphe_galaxies.construction_graphe(database_path=path_file_galaxies,
                                                   step_time=step_nodes_time)
    t2 = time.clock()
    print("Temps de construction du graphe: " + str(t2 - t1) + " secondes")

    # Sauvegarde du graphe
    t1 = time.clock()
    graphe_galaxies.sauvegarde_graphe(database_path=path_file_galaxies,
                                      step_time=step_nodes_time)
    t2 = time.clock()
    print("Temps de sauvegarde du graphe: " + str(t2 - t1) + " secondes")

    if maxNoeud == 0:
        maxNoeud = database_tools.max_nodes(path_file_galaxies)

    # Extraction des composantes connexes
    t1 = time.clock()
    extraction_galaxies.extraction_composantes_connexes_(path_file_galaxies=path_file_galaxies,
                                                         path_file_list_galaxies=path_file_list_galaxies,
                                                         maxNoeud=maxNoeud,
                                                         step_number_of_nodes_galaxie=step_number_of_nodes_galaxie)
    t2 = time.clock()
    print("Temps total d'extraction des composantes connexes:", t2 - t1, "sec.")


# TODO: Supprimer toutes les methodes ci-dessous
def impressionTexteGUI(n, p, path):
    res = "Composante " + n + ":\n\n"
    q = int(p)
    L = extraction_galaxies.textes_galaxie(n, path)
    while L != set() and (q > 0 or p == 0):
        E = L.pop()
        res += "- " + str(E) + "\n"
        q -= 1
    return res


def extractionComposantes():
    maxNoeud = database_tools.max_nodes()
    t1 = time.clock()
    extraction_galaxies.extraction_composantes_connexes_(maxNoeud)
    t2 = time.clock()
    print("Temps total d'extraction des composantes connexes: " +
          str(t2 - t1) + "sec.")


def impressionTexte(numero_composante, p):
    q = p
    L = extraction_galaxies.textes_galaxie(numero_composante)

    while L != set() and (q > 0 or p == 0):
        E = L.pop()
        print("- " + str(E))
        q -= 1


def impressionTexteLongueur(numero_composante, p):
    q = p
    L = sorted(extraction_galaxies.textes_galaxie(numero_composante),
               key=lambda reference: len(reference))

    while L != [] and (q > 0 or p == 0):
        E = L.pop()
        print("- " + str(E))
        q -= 1


def impressionTexteEtReferenceLongueur(numero_composante, p):
    q = p
    L = sorted(extraction_galaxies.textes_et_references_galaxie(numero_composante),
               key=lambda reference: len(reference[0]))

    while L != [] and (q > 0 or p == 0):
        E = L.pop()
        print("- " + str(E[0]) + " références: " + str(E[1]))
        q -= 1


def impressionTexteEtReference(numero_composante, p):
    q = p
    L = extraction_galaxies.textes_et_references_galaxie(numero_composante)

    while L != set() and (q > 0 or p == 0):
        E = L.pop()
        print("- " + str(E[0]) + " références: " + str(E[1]))
        q -= 1


def impressionTexteEtReferenceAnciennete(numero_composante, p):
    q = p
    L = sorted(extraction_galaxies.textes_et_references_galaxie(numero_composante),
               key=lambda reference: str(reference[1][2]))

    while L != [] and (q > 0 or p == 0):
        E = L[0]
        L = L[1:]
        print("- " + str(E[0]) + " références: " + str(E[1]))
        q -= 1


if __name__ == '__main__':
    pass



# -*- coding: utf-8 -*-
""" Ce module contient des fonctions gérant l'accès, la création et la
suppression dans la base de données SQLite3.
"""
import sqlite3
import os.path


DATABASE_NAME = "galaxie.db"

# Files containing SQLite3 statements
SQL_CREATION_GALAXIE = os.path.join(os.path.dirname(__file__),
                                    "../../res/sql/create_galaxie.sql")


def connexion(database_path: str) -> sqlite3.Connection:
    """ Effectue une connexion vers la base de données """
    return sqlite3.connect(database_path, 1, 0, 'EXCLUSIVE')


def create_database(database_path: str) -> None:
    with open(SQL_CREATION_GALAXIE) as file:
        statement = file.read()

    cursor = connexion(database_path).cursor()
    cursor.executescript(statement)
    cursor.close()


def add_book(auteur, titre, date, cursor):
    """ Ajoute un livre dans la base de données. """
    statement = """
        INSERT INTO livres values (?,?,?,?)
    """
    id_book = id_ref(auteur, titre, date)

    try:
        cursor.execute(statement, (id_book, auteur, titre, date))
    # Don't add books already in the database
    except sqlite3.IntegrityError:
        pass


def id_ref(auteur, titre, date):
    """ Retourne un ID formaté utilisé dans la base de données. """
    return auteur + titre + str(date)


def get_id_book(auteur, titre, date, cursor):
    """ Retourne l'ID d'un livre depuis la base de données. """
    query = """
        SELECT rowid
        FROM livres
        WHERE idLivre = (?)
    """
    cursor.execute(query, (id_ref(auteur, titre, date),))
    return cursor.fetchone()[0]


def add_reuse(id_source, coordonnee_source, empan_source, texte_source,
              id_cible, coordonnee_cible, empan_cible, texte_cible,
              real_id_book_source, real_id_book_target, cursor):
    """ Ajoute une réutilisation dans la base de données. """
    statement = '''
        INSERT INTO grapheReutilisations values (?,?,?,?,?,?,?,?,?,?)
    '''
    cursor.execute(statement, (id_source, coordonnee_source, empan_source,
                               texte_source, id_cible, coordonnee_cible,
                               empan_cible, texte_cible, real_id_book_source,
                               real_id_book_target))


def max_nodes(database_path):
    """ Recupère l'ID qui apparait dans le plus de noeuds. """
    query = '''
        SELECT *
        FROM maxNoeud
    '''

    curseur = connexion(database_path).cursor()
    curseur.execute(query)
    return curseur.fetchone()[0]


if __name__ == '__main__':
    pass



# -*- coding: utf-8 -*-
""" Contient les différents arguments passés en entrée et utilisés par les
modules `main.py` et `galaxie_creation.py`.
Une valeur assignée directement dans ce code correspond à une valeur par défaut.
"""

import os.path


######################
## Needed arguments ##
######################
# Path of the source database, containing all reuses.
# The database must be in the TSV, TAB or CSV format.
path_database_source = None


#########################
## Optional parameters ##
#########################
# Max book to read from the source file and to add in the database.
# Set to -1 for all books available
max_book_to_add = -1

# Filenames created by the software
file_name_galaxies = "galaxies.db"
file_name_list_galaxies = "list_galaxies.db"
file_name_adjacency_graph = "ajacency_graph.db"
file_name_adjacency_graph_transposed = "ajacency_graph_transposed.db"

# Path for directories and files
path_dir_dest = "results"  # Directory containing all created files
path_file_galaxies = os.path.join(path_dir_dest, file_name_galaxies)
path_file_list_galaxies = os.path.join(path_dir_dest, file_name_list_galaxies)
path_file_adjacency_graph = os.path.join(path_dir_dest, file_name_adjacency_graph)
path_file_adjacency_graph_transposed = os.path.join(path_dir_dest, file_name_adjacency_graph_transposed)

# Delimiter used to pass the reuses' file
delimiter = "\t"

# Encoding of the reuses' file
encoding_src = "utf8"

# Remove the header present in the reuses' file
remove_header = True

# Indexes of the data which is gonna ba extracted from the reuses' file
index_src_author = 2
index_src_title = 2
index_src_date = 12
index_src_text_matched = 6
index_src_start_byte = 3
index_src_end_byte = 4

index_dest_author = 15
index_dest_title = 16
index_dest_date = 12
index_dest_text_matched = 20
index_dest_start_byte = 17
index_dest_end_byte = 18

# Affiche le temps de construction tous les `step_nodes_time` noeuds effectués
step_nodes_time = 10000
step_galaxies_time = 10000
step_number_of_nodes_galaxie = 10000


if __name__ == '__main__':
    pass



import gi
gi.require_version('Gtk', '3.0')

from gi.repository import Gtk, Gdk

from galaxies.src.gui.gui_tools import *


WINDOW_TITLE = "Galaxie Vizualisation"


class WindowsGalaxieVizualisation:
    def __init__(self):
        self.list_filter = []
        self.list_button = []
        self.path = None
        self.window = Gtk.Window(title=WINDOW_TITLE)
        self.window.maximize()

        # self.window.set_position(Gtk.WIN_POS_CENTER)

        # self.grid = Gtk.Grid()

        self.menubar = Gtk.MenuBar()

        self.menu1 = Gtk.Menu()
        self.menu1_file = Gtk.MenuItem("Fichier")
        self.menu1_file.set_submenu(self.menu1)

        self.menu2 = Gtk.Menu()
        self.menu2_filtre = Gtk.MenuItem("Filtre")
        self.menu2_filtre.set_submenu(self.menu2)

        self.menu3 = Gtk.Menu()
        self.menu3_aide = Gtk.MenuItem("Aide")
        self.menu3_aide.set_submenu(self.menu2)

        self.charger_reutilisation = Gtk.MenuItem("Charger Réutilisation")
        self.charger_reutilisation.connect('activate', self.charger_reu)
        self.menu1.append(self.charger_reutilisation)
        self.charger_graphe = Gtk.MenuItem("Charger Graphe")
        self.menu1.append(self.charger_graphe)
        self.menu1.append(Gtk.SeparatorMenuItem())
        self.save_graphe = Gtk.MenuItem("Sauvegarder Graphe")
        self.menu1.append(self.save_graphe)
        self.menu1.append(Gtk.SeparatorMenuItem())
        self.exit = Gtk.MenuItem("Exit")
        self.exit.connect('activate', Gtk.main_quit)
        self.menu1.append(self.exit)

        self.filtre_auteur = Gtk.MenuItem("Filtrer Auteur")
        self.filtre_auteur.connect('activate',
                                   lambda x=0, y=0: self.filtre(x, y))
        self.menu2.append(self.filtre_auteur)
        self.filtre_annee = Gtk.MenuItem("Filtrer Année")
        self.filtre_annee.connect('activate',
                                  lambda x=0, y=1: self.filtre(x, y))
        self.menu2.append(self.filtre_annee)

        self.menubar.append(self.menu1_file)
        self.menubar.append(self.menu2_filtre)
        self.menubar.append(self.menu3_aide)

        self.vbox = Gtk.VBox(False, 2)
        self.vbox.pack_start(self.menubar, False, False, 0)
        self.window.add(self.vbox)

        self.hbox = Gtk.HBox(False, 0)

        self.sw = Gtk.ScrolledWindow()
        self.sw2 = Gtk.ScrolledWindow()

        self.vbox2 = Gtk.VBox(False, 2)

        # self.but = Gtk.Button(label="Filtre 1\nFiltre Auteur: Shakespear")
        # self.but2 = Gtk.Button(label="Filtre 2\nFiltre Auteur: Flaubert")
        #		self.but3 = Gtk.Button(label="Filtre 3\nFiltre Auteur: Maupassant")

        #	self.but4 = Gtk.Button(label="Filtre 3\nFiltre Auteur: Shakespear")
        #	self.but5 = Gtk.Button(label="Filtre 4\nFiltre Auteur: Flaubert")
        #	self.but6 = Gtk.Button(label="Filtre 5\nFiltre Auteur: Maupassant")

        #	self.but7 = Gtk.Button(label="Filtre 6\nFiltre Auteur: Shakespear#")
        #	self.but8 = Gtk.Button(label="Filtre 7\nFiltre Auteur: Flaubert")
        #	self.but9 = Gtk.Button(label="Filtre 8\nFiltre Auteur: Maupassant")

        # self.label = Gtk.Label(label="BlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablabl\nablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablabla\nBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablablaBlablablabla")

        self.g = Graph()

        position = sfdp_layout(self.g)
        self.l = GraphWidget(self.g, pos=position, vertex_font_size=15)

        self.l.set_size_request(self.window.get_screen().get_width() - 200,
                                self.window.get_screen().get_height() - 400)
        # self.vbox2.pack_start(self.but,  False, True, 5)
        # self.vbox2.pack_start(self.but2, False, True, 5)
        # self.vbox2.pack_start(self.but3, False, True, 5)

        # self.vbox2.pack_start(self.but4, False, True, 5)
        # self.vbox2.pack_start(self.but5, False, True, 5)
        # self.vbox2.pack_start(self.but6, False, True, 5)

        # self.vbox2.pack_start(self.but7, False, True, 5)
        # self.vbox2.pack_start(self.but8, False, True, 5)
        # self.vbox2.pack_start(self.but9, False, True, 5)

        self.sw.add_with_viewport(self.vbox2)
        self.hbox.pack_start(self.sw, True, True, 0)
        self.hbox.pack_start(self.l, False, False, 0)

        color = Gdk.color_parse('#d3d3d3')
        color2 = Gdk.color_parse('#595959')
        self.sw.modify_bg(Gtk.StateType.NORMAL, color)
        self.sw2.modify_bg(Gtk.StateType.NORMAL, color)
        # self.sw2.add_with_viewport(self.label)

        self.vbox.add(self.hbox)
        self.vbox.add(self.sw2)

        self.window.connect("delete-event", Gtk.main_quit)
        self.window.show_all()

        Gtk.main()

    def charger_reu(self, event):
        dialog = Gtk.FileChooserDialog("Please choose a file", self.window,
                                       Gtk.FileChooserAction.OPEN,
                                       (Gtk.STOCK_CANCEL,
                                        Gtk.ResponseType.CANCEL,
                                        Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
        # self.add_filters(dialog)
        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            self.path = dialog.get_filename()
            maxi, rows = get_father_son(self.path)
            self.redraw_graphe(maxi[0] + 1, rows)
        dialog.destroy()

    def redraw_graphe(self, number_nodes, list_edges):
        tab = [self.g.add_vertex() for i in range(number_nodes)]
        tab2 = [self.g.add_edge(tab[edge[0]], tab[edge[1]]) for edge in
                list_edges]
        self.l.destroy()
        position = sfdp_layout(self.g)
        self.l = GraphWidget(self.g, pos=position, vertex_font_size=15)
        self.l.set_size_request(self.window.get_screen().get_width() - 200,
                                self.window.get_screen().get_height() - 400)
        self.hbox.pack_start(self.l, False, False, 0)
        self.window.show_all()

    def filtre(self, event, choix):
        if self.path is not None:
            author, date = get_authors_date(self.path)
            dialog = Gtk.Dialog("Filtre", self.window, 0, (
            Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OK,
            Gtk.ResponseType.OK))
            v = dialog.get_content_area()

            if choix == 0:
                type_filter = "auteur:"
                liste = Gtk.ListStore(str)
                for i in author:
                    liste.append([i[0]])
            else:
                type_filter = "année:"
                liste = Gtk.ListStore(int)
                for i in date:
                    liste.append([i[0]])

            cb = Gtk.ComboBox().new_with_model(liste)
            rt = Gtk.CellRendererText()
            cb.pack_start(rt, True)
            cb.add_attribute(rt, "text", 0)

            check_b = Gtk.CheckButton("Inverser")
            v.pack_start(cb, False, False, 0)
            v.pack_start(check_b, False, False, 0)

            dialog.show_all()
            response = dialog.run()
            if response == Gtk.ResponseType.OK:
                i = cb.get_active_iter()
                if i is not None:
                    if check_b.get_active():
                        inv = "inversé "
                    else:
                        inv = " "
                    fil = cb.get_model()[i][0]
                    self.list_filter.append(Gtk.VBox(False, 2))
                    self.list_filter[-1].set_border_width(5)
                    self.list_filter[-1].override_background_color(
                        Gtk.StateFlags.NORMAL, Gdk.RGBA(255, 255, 255))
                    self.list_filter[-1].pack_start(
                        Gtk.Label("Filtre " + inv + type_filter), False, False,
                        0)
                    self.list_filter[-1].pack_start(Gtk.Label(fil), False,
                                                    False, 0)
                    self.list_button.append(
                        Gtk.Button.new_with_label("Supprimer"))
                    self.list_filter[-1].pack_start(self.list_button[-1], False,
                                                    False, 0)
                    self.list_button[-1].connect("clicked",
                                                 self.supprimer_filtre)
                    self.vbox2.pack_start(self.list_filter[-1], False, False, 0)
                    self.window.show_all()

            dialog.destroy()

    def supprimer_filtre(self, button):
        i = self.list_button.index(button)
        self.list_button.pop(i)
        vbox_to_del = self.list_filter.pop(i)
        vbox_to_del.destroy()
        self.window.show_all()


if __name__ == "__main__":
    pass



""" This modules contains all functions, methods and classes related to GUI. """
from tkinter import *
from tkinter.filedialog import *
from tkinter.messagebox import *

import galaxies.src.core.environment as env
from galaxies.src.core.interface import create_reuses_database, \
    impressionTexteGUI


# Theme-related variables
DARK = 0
GRAY = 1
TURQUOISE = 2
BLUE = 3
THEMES = {
    DARK: {
        "BG_BUTTON": '#%02x%02x%02x' % (94, 94, 94),
        "BG_FRAME": '#%02x%02x%02x' % (68, 68, 68),
        "BG_ENTRY": '#%02x%02x%02x' % (42, 42, 42),
        "FG_COLOR": "white"
    },
    GRAY: {
        "BG_BUTTON": '#%02x%02x%02x' % (240, 240, 240),
        "BG_FRAME": '#%02x%02x%02x' % (240, 240, 240),
        "BG_ENTRY": "white",
        "FG_COLOR": "black"
    },
    TURQUOISE: {
        "BG_BUTTON": '#%02x%02x%02x' % (1, 133, 134),
        "BG_FRAME": '#%02x%02x%02x' % (1, 133, 134),
        "BG_ENTRY": '#%02x%02x%02x' % (0, 86, 87),
        "FG_COLOR": "white"
    },
    BLUE: {
        "BG_BUTTON": '#%02x%02x%02x' % (0, 169, 232),
        "BG_FRAME": '#%02x%02x%02x' % (0, 169, 232),
        "BG_ENTRY": '#%02x%02x%02x' % (199, 224, 255),
        "FG_COLOR": '#%02x%02x%02x' % (11, 38, 75)
    }
}

# Messages-related variables
MESSAGE_ERROR_FORMAT = "Formats currently supported are : {}."
MESSAGE_ERROR_NO_SOURCE_FILE = "No reuses' file chosen."

LISTE_PARAM1 = [env.index_src_author, env.index_src_title, env.index_src_date,
                env.index_src_text_matched, env.index_src_start_byte,
                env.index_src_end_byte, env.index_dest_author, env.index_dest_title,
                env.index_dest_date, env.index_dest_text_matched, env.index_dest_start_byte,
                env.index_dest_end_byte]
LISTE_PARAM2 = [env.step_nodes_time, env.step_galaxies_time, env.step_number_of_nodes_galaxie]

POSSIBLES_EXTENSIONS = ("tsv", "tab", "csv")
FILETYPES = [('tsv files', '.tsv'),
             ('tab files', '.tab'),
             ('csv files', '.csv'),
             ('all files', '.*')]


class WindowsGalaxieCreation:
    def __init__(self, theme=GRAY):
        self.BG_BUTTON = None
        self.BG_FRAME = None
        self.BG_ENTRY = None
        self.FG_COLOR = None
        self._set_colors(theme)

        # Création de la fenêtre
        self.window = Tk()
        self.window_frame = Frame(self.window, bg=self.BG_FRAME)
        self.window.resizable(False, False)
        self.window_frame.grid()

        self.all_button = []
        self.all_entry = []
        self.all_label = []
        self.all_frame = []

        self.run_already = False

        self._place_menu_bar()
        self._create_bindings()
        self.panel_create()

        self.window.mainloop()

    def panel_open_database(self, event=None):
        self.clear_frame(self.window_frame)
        self.position = 1
        path = askopenfilename(title="Open", filetypes=FILETYPES)
        if path == "":
            self.panel_create()
        if path[-3:] in POSSIBLES_EXTENSIONS:
            with open(path, 'r') as T:
                self.liste_line = [next(T) for i in range(100)]
            header = self.liste_line[0]
            text = self.liste_line[1]

            self.label_head = Label(self.window_frame, text="Header", bg=self.BG_FRAME, fg=self.FG_COLOR)
            self.all_label.append(self.label_head)
            self.header = Text(self.window_frame, width=100, height=6, bg=self.BG_ENTRY, fg=self.FG_COLOR)
            self.all_entry.append(self.header)
            self.label_text = Label(self.window_frame, text="File", bg=self.BG_FRAME, fg=self.FG_COLOR)
            self.all_label.append(self.label_text)
            self.text = Text(self.window_frame, width=100, height=15, bg=self.BG_ENTRY, fg=self.FG_COLOR)
            self.all_entry.append(self.text)

            self.previous_button = Button(self.window_frame, text="Prev", bg=self.BG_BUTTON, fg=self.FG_COLOR, state=DISABLED, command=lambda x=-1 : self.next_previous(x))
            self.next_button = Button(self.window_frame, text="Next", bg=self.BG_BUTTON, fg=self.FG_COLOR, command=lambda x=1 : self.next_previous(x))
            self.create_button = Button(self.window_frame, text="Create database", bg=self.BG_BUTTON, fg=self.FG_COLOR, command=self.create_reuses_database)

            self.all_button.append(self.previous_button)
            self.all_button.append(self.next_button)
            self.all_button.append(self.create_button)

            self.header.insert(1.0, header)
            self.text.insert(1.0, text)

            self.label_head.grid(row=1, column=1, columnspan=2)
            self.header.grid(row=2, column=1, columnspan=2)
            self.label_text.grid(row=3, column=1, columnspan=2)
            self.text.grid(row=4, column=1, columnspan=2)
            self.previous_button.grid(row=5, column=1, sticky=E)
            self.next_button.grid(row=5, column=2, sticky=W)
            self.create_button.grid(row=6, column=1, columnspan=2, pady=15)
        else:
            showerror("Error", "Only tsv file")
            self.panel_create()

    def next_previous(self, n):
        self.position += n
        if self.position == 1:
            self.previous_button.config(state=DISABLED)
        elif self.position == 2:
            self.previous_button.config(state=NORMAL)
        elif self.position == 99:
            self.next_button.config(state=DISABLED)
        elif self.position == 98 and n == -1:
            self.next_button.config(state=NORMAL)

        self.text.delete('1.0', END)
        self.text.insert(1.0, self.liste_line[self.position])

    def panel_create(self, event=None, option=True):
        self.clear_frame(self.window_frame)
        self.path_open = StringVar()
        self.path_save = StringVar()
        self.create_frame = LabelFrame(self.window_frame, text="Create Database", bg=self.BG_FRAME, fg=self.FG_COLOR)
        self.all_frame.append(self.create_frame)
        self.create_frame.grid(row=1, padx=5, pady=5)
        self.button_open = Button(self.create_frame, text=" Open  ", bg=self.BG_BUTTON, fg=self.FG_COLOR, command=self.open_reuses_database)
        self.all_button.append(self.button_open)
        self.entry_open = Entry(self.create_frame, textvariable=self.path_open, width=30, bg=self.BG_ENTRY, fg=self.FG_COLOR)
        #self.entry_save_as = Entry(self.create_frame, textvariable=self.path_save, width=30, bg=self.BG_ENTRY, fg=self.FG_COLOR)
        self.all_entry.append(self.entry_open)
        #self.all_entry.append(self.entry_save_as)
        #self.button_save_as = Button(self.create_frame, text="Save as", bg=self.BG_BUTTON, fg=self.FG_COLOR, command=self.save_create)
        self.button_create = Button(self.create_frame, text="Create", bg=self.BG_BUTTON, fg=self.FG_COLOR, command=self.create_reuses_database)
        #self.all_button.append(self.button_save_as)
        self.all_button.append(self.button_create)

        self.button_open.grid(row=1, column=1, padx=(5, 0), pady=5)
        self.entry_open.grid(row=1, column=2, padx=(0, 5), pady=5)
        #self.button_save_as.grid(row=2, column=1, padx=(5,0), pady=5)
        #self.entry_save_as.grid(row=2, column=2, padx=(0,5), pady=5)
        self.button_create.grid(row=3, column=1, columnspan=2, padx=5, pady=5)

        if option:
            self.panel_run()

    def panel_run(self, event=None, row=8):
        self.run_frame = LabelFrame(self.window_frame, text="Run",
                                    bg=self.BG_FRAME, fg=self.FG_COLOR)
        self.button_open2 = Button(self.run_frame, text=" Open  ",
                                   bg=self.BG_BUTTON, fg=self.FG_COLOR, command=self.open_galaxies_database)
        self.path_database = StringVar()
        self.entry_open2 = Entry(self.run_frame,
                                 textvariable=self.path_database, width=30,
                                 bg=self.BG_ENTRY, fg=self.FG_COLOR)
        self.button_run = Button(self.run_frame, text="Run",
                                 bg=self.BG_BUTTON, fg=self.FG_COLOR, command=self.run)

        self.label_n = Label(self.run_frame, text="n", bg=self.BG_FRAME, fg=self.FG_COLOR)
        self.all_label.append(self.label_n)
        self.label_p = Label(self.run_frame, text="p", bg=self.BG_FRAME, fg=self.FG_COLOR)
        self.all_label.append(self.label_p)
        self.spin_n = Spinbox(self.run_frame, width=5, bg=self.BG_ENTRY, fg=self.FG_COLOR, insertbackground="white", buttonbackground=self.BG_BUTTON)
        self.spin_p = Spinbox(self.run_frame, width=5, bg=self.BG_ENTRY, fg=self.FG_COLOR, insertbackground="white", buttonbackground=self.BG_BUTTON)
        self.all_spin = [self.spin_n, self.spin_p]

        self.run_frame.grid(row=row, columnspan=3, padx=5, pady=5)
        self.button_open2.grid(row=1, column=1, padx=(5, 0), pady=5)
        self.entry_open2.grid(row=1, column=2, columnspan=3, padx=(0, 5), pady=5)
        self.button_run.grid(row=3, column=1, columnspan=4, padx=5, pady=5)

        self.label_n.grid(row=2, column=1, padx=(0,5), sticky=E)
        self.label_p.grid(row=2, column=3, padx=(0,5), sticky=E)
        self.spin_n.grid(row=2, column=2, pady=(0,10), sticky=W)
        self.spin_p.grid(row=2, column=4, pady=(0,10), sticky=W)

        self.all_frame.append(self.run_frame)
        self.all_button.append(self.button_open2)
        self.all_button.append(self.button_run)
        self.all_entry.append(self.entry_open2)

    def panel_edit_parameters(self, event=None, option=2):
        self.parameters_window = Tk()
        self.parameters_window.title("Edit Parameters")
        self.parameters_window.resizable(width=False, height=False)
        self.parameters_window.config(bg=self.BG_FRAME)
        self.header_frame = LabelFrame(self.parameters_window,
                                       text="Header Parameters",
                                       bg=self.BG_FRAME, fg=self.FG_COLOR)
        self.step_frame = LabelFrame(self.parameters_window,
                                     text="Step Parameters", bg=self.BG_FRAME,
                                     fg=self.FG_COLOR)

        self.all_frame.append(self.header_frame)
        self.all_frame.append(self.step_frame)

        self.save_button = Button(self.parameters_window, text="Save",
                                  bg=self.BG_BUTTON, fg=self.FG_COLOR, command=lambda x=option:self.save_param(x))
        self.save_button.grid(row=3, pady=5)

        self.all_button.append(self.save_button)

        tab_string1 = ["Source Author", "Source Title", "Source Date",
                       "Source Match", "Source Start Byte",
                       "Source End Byte", "Source End Byte",
                       "Target Title", "Target Date", "Target Match",
                       "Target Start Byte", "Target End Byte"]

        tab_string2 = ["Step Number Node",
                       "Step Galaxies", "Step Number Nodes Galaxie"]

        self.list_label_header = [Label(self.header_frame, text=label, bg=self.BG_FRAME, fg=self.FG_COLOR) for label in tab_string1]
        self.list_spin_header = [Spinbox(self.header_frame, width=5, bg=self.BG_ENTRY, fg=self.FG_COLOR, insertbackground="white", buttonbackground=self.BG_BUTTON) for _ in tab_string1]
        self.list_label_step = [Label(self.step_frame, text=label, bg=self.BG_FRAME, fg=self.FG_COLOR) for label in tab_string2]
        self.list_spin_step = [Spinbox(self.step_frame, width=10, bg=self.BG_ENTRY, fg=self.FG_COLOR, insertbackground="white", buttonbackground=self.BG_BUTTON) for _ in tab_string2]

        for i in range(len(tab_string1)):
            if i < len(LISTE_PARAM2):
                self.list_spin_step[i].insert(0, LISTE_PARAM2[i])
            self.list_spin_header[i].insert(0, LISTE_PARAM1[i])

        for label in self.list_label_header:
            self.all_label.append(label)

        for label in self.list_label_step:
            self.all_label.append(label)

        if option in [0, 2]:
            self.place_frame(self.header_frame, 1, self.list_label_header,
                             self.list_spin_header, 3)

        if option in [1, 2]:
            self.place_frame(self.step_frame, 2, self.list_label_step,
                             self.list_spin_step, 2)

        self.parameters_window.mainloop()

    def save_param(self, option):
        try:
            if option in [0,2]:
                sourceAuthor = int(self.list_spin_header[0].get())
                sourceTitle = int(self.list_spin_header[1].get())
                sourceDate = int(self.list_spin_header[2].get())
                sourceMatch = int(self.list_spin_header[3].get())
                sourceStartByte = int(self.list_spin_header[4].get())
                sourceEndByte = int(self.list_spin_header[5].get())
                targetAuthor = int(self.list_spin_header[6].get())
                targetTitle = int(self.list_spin_header[7].get())
                targetDate = int(self.list_spin_header[8].get())
                targetMatch = int(self.list_spin_header[9].get())
                targetStartByte = int(self.list_spin_header[10].get())
                targetEndByte = int(self.list_spin_header[11].get())

            if option in [1,2]:
                pasTracage = int(self.list_spin_step[0].get())
                pasNbreNoeud = int(self.list_spin_step[1].get())
                pasGalaxies = int(self.list_spin_step[2].get())
                pasNbreNoeudsGalaxie = int(self.list_spin_step[3].get())

            self.parameters_window.destroy()
        except:
            showerror("Error", "Only integers")

    def place_frame(self, frame, row_frame, list_label, list_spin, n_column):
        frame.grid(row=row_frame, padx=5, pady=5)
        row = 1
        column = 1
        for i in range(len(list_label)):
            if i % n_column == 0:
                row += 1
                column = 1

            list_label[i].grid(row=row, column=column, padx=(0, 5), sticky=W)
            list_spin[i].grid(row=row, column=column + 1, padx=(0, 5),
                              pady=(0, 2), sticky=E)
            column += 2

    def clear_frame(self, frame):
        for widget in frame.winfo_children():
            if widget in self.all_frame:
                self.all_frame.remove(widget)
            elif widget in self.all_button:
                self.all_button.remove(widget)
            elif widget in self.all_entry:
                self.all_entry.remove(widget)
            elif widget in self.all_label:
                self.all_label.remove(widget)
            widget.destroy()

    def change_theme(self, theme):
        """ Change the theme of the Window.
        First, it change the attributes containing the colors of the window,
        then it reload all its components to set the new colors.
        """
        self._set_colors(theme)
        self._reload_colors()

    def _set_colors(self, theme):
        """ Change the attributes containing the colors of the window. """
        for name, color in zip(THEMES[theme].keys(), THEMES[theme].values()):
            setattr(self, name, color)

    def _reload_colors(self):
        """ Reload all the components of the windows with their the new color.
        """
        self.window_frame.config(bg=self.BG_FRAME)
        self.menu_bar.config(bg=self.BG_ENTRY)
        for button in self.all_button:
            try:
                button.config(bg=self.BG_BUTTON, fg=self.FG_COLOR)
            except:
                pass
        for entry in self.all_entry:
            try:
                entry.config(bg=self.BG_ENTRY, fg=self.FG_COLOR)
            except:
                pass
        for frame in self.all_frame:
            try:
                frame.config(bg=self.BG_FRAME, fg=self.FG_COLOR)
            except:
                pass
        for label in self.all_label:
            try:
                label.config(bg=self.BG_FRAME, fg=self.FG_COLOR)
            except:
                pass
        for spin in self.all_spin:
            try:
                spin.config(bg=self.BG_ENTRY, fg=self.FG_COLOR, buttonbackground=self.BG_BUTTON)
            except:
                pass

    def open_reuses_database(self):
        path = askopenfilename(title="Open", filetypes=FILETYPES)
        if len(path) == 0:
            # No path chosen
            return
        elif path[-3:] in POSSIBLES_EXTENSIONS:
            # Path chosen in possibles extensions
            self.path_open.set(path)
        else:
            # Path chosen but not supported extension
            showerror("Wrong file format",
                      MESSAGE_ERROR_FORMAT.format(", "
                                                  .join(POSSIBLES_EXTENSIONS)))

    def open_galaxies_database(self):
        print("CALLED open_galaxies_database")
        # TODO: Delete this method. The format should not be asked with a window but should be set has a param (and defaulting to the value "galaxies.db")
        path = askopenfilename(title="Open", filetypes=FILETYPES)
        self.path_database.set(path)

    def create_reuses_database(self):
        print("CALLED create_reuses_database")
        path = env.file_name_galaxies
        # TODO: Check if filepath is valid
        self.path_save.set(path)
        if self.path_open.get() != "":
            create_reuses_database(database_src=self.path_open.get(),
                                   path_file_galaxies=os.path.join(env.path_dir_dest, self.path_save.get()),
                                   path_file_list_galaxies=env.path_file_list_galaxies,
                                   path_file_adjacency_graph=env.path_file_adjacency_graph,
                                   path_file_adjacency_graph_transposed=env.path_file_adjacency_graph_transposed,
                                   path_dir_dest=env.path_dir_dest,
                                   max_book_to_add=env.max_book_to_add,
                                   delimiter=env.delimiter,
                                   encoding_src=env.encoding_src,
                                   remove_header=env.remove_header,
                                   index_src_author=env.index_src_author,
                                   index_src_title=env.index_src_title,
                                   index_src_date=env.index_src_date,
                                   index_src_text_matched=env.index_src_text_matched,
                                   index_src_start_byte=env.index_src_start_byte,
                                   index_src_end_byte=env.index_src_end_byte,
                                   index_dest_author=env.index_dest_author,
                                   index_dest_title=env.index_dest_title,
                                   index_dest_date=env.index_dest_date,
                                   index_dest_text_matched=env.index_dest_text_matched,
                                   index_dest_start_byte=env.index_dest_start_byte,
                                   index_dest_end_byte=env.index_dest_end_byte,
                                   step_nodes_time=env.step_nodes_time,
                                   step_galaxies_time=env.step_galaxies_time,
                                   step_number_of_nodes_galaxie=env.step_number_of_nodes_galaxie)
        else:
            showerror("No source file",
                      MESSAGE_ERROR_NO_SOURCE_FILE)

    def run(self):
        if self.run_already:
            self.label_res.destroy()
        if self.path_database.get() != "":
            n = self.spin_n.get()
            p = self.spin_p.get()
            # TODO: Faire ça propre
            res = impressionTexteGUI(n, p, self.path_database.get())
            self.label_res = Label(self.run_frame, text=res, bg=self.BG_FRAME, fg=self.FG_COLOR)
            self.label_res.grid(row=5, column=1, columnspan=4, pady=10, sticky=W)
            self.run_already = True

    def _place_menu_bar(self):
        self.menu_bar = Menu(self.window)

        # Create the menu file
        self.menu_file = Menu(self.menu_bar, tearoff=0)
        self.menu_file.add_command(label="Open", accelerator="Ctrl+O", command=self.panel_open_database)
        self.menu_file.add_separator()
        self.menu_file.add_command(label="Create", accelerator="Ctrl+C", command=lambda x=0, y=False : self.panel_create(x, y))
        self.menu_file.add_command(label="Run", accelerator="Ctrl+R", command=self.panel_run)
        self.menu_file.add_separator()
        self.menu_file.add_command(label="Quit", accelerator="Ctrl+Q", command=self.window.quit)

        # Create the menu parameters
        self.menu_parameters = Menu(self.menu_bar, tearoff=0)
        self.menu_parameters.add_command(label="Edit all", accelerator="Ctrl+P", command=self.panel_edit_parameters)
        self.menu_parameters.add_command(label="Edit step", accelerator="Ctrl+S", command=lambda x=0, y=1 :self.panel_edit_parameters(x, y))
        self.menu_parameters.add_command(label="Edit heading", accelerator="Ctrl+H", command=lambda x=0, y=0 :self.panel_edit_parameters(x, y))

        # Create the menu theme
        self.menu_theme = Menu(self.menu_bar, tearoff=0)
        self.menu_theme.add_command(label="Blue", command=lambda x=BLUE : self.change_theme(x))
        self.menu_theme.add_command(label="Dark", command=lambda x=DARK : self.change_theme(x))
        self.menu_theme.add_command(label="Gray", command=lambda x=GRAY : self.change_theme(x))
        self.menu_theme.add_command(label="Turquoise", command=lambda x=TURQUOISE : self.change_theme(x))

        # Assign all menus to the menu bar
        self.menu_bar.add_cascade(label="Database", menu=self.menu_file)
        self.menu_bar.add_cascade(label="Parameters", menu=self.menu_parameters)
        self.menu_bar.add_cascade(label="Theme", menu=self.menu_theme)
        self.window.config(menu=self.menu_bar)

    def _create_bindings(self):
        self.window.bind_all("<Control-q>", quit)
        self.window.bind_all("<Control-o>", self.panel_open_database)
        self.window.bind_all("<Control-p>", self.panel_edit_parameters)
        self.window.bind_all("<Control-r>", self.panel_run)
        self.window.bind_all("<Control-s>", lambda x=0, y=1 :self.panel_edit_parameters(x, y))
        self.window.bind_all("<Control-h>", lambda x=0, y=0 :self.panel_edit_parameters(x, y))
        self.window.bind_all("<Control-c>", lambda x=0, y=False :self.panel_create(x, y))

    def quit(self):
        self.window.destroy()


if __name__ == '__main__':
    pass



""" A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""
from setuptools import setup, find_packages

from codecs import open
from os import path

import galaxies.src.getters.get_global_variable as ggv


_HERE = path.abspath(path.dirname(__file__))
_README_FILE_NAME = "README.md"
_README_FILE_ENCODING = "utf8"


# Get the long description from the README file
with open(path.join(_HERE, _README_FILE_NAME),
          encoding=_README_FILE_ENCODING) as file:
    long_description = file.read()

setup(
    name=ggv.name(),

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=ggv.version(),

    description=ggv.description(),
    long_description=long_description,

    # The project's main homepage.
    url=ggv.main_homepage(),

    # Not to use with Python versions prior to 2.2.3 or 2.3
    download_url=ggv.download_url(),

    # Author details
    author=ggv.author(),
    author_email=ggv.email(),

    # Choose your license
    license=ggv.license_used(),

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=ggv.classifiers(),

    # What does your project relate to?
    keywords=ggv.keywords(),

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=ggv.dependencies(),

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={

    },

    # Set to True if we use MANIFEST.in
    include_package_data=True,

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            ggv.cli_entry_point() + ' = galaxies.main:cli_entry_point',
        ],
        'gui_scripts': [
            ggv.gui_entry_point() + ' = galaxies.main:gui_entry_point'
        ]
    }
)



# -*- coding: utf-8 -*-
from galaxies.src.gui.main_menu import WindowMainMenu


def cli_entry_point():
    print("Hello CLI !")


def gui_entry_point():
    WindowMainMenu()


if __name__ == '__main__':
    gui_entry_point()
